
----------
March 2016 V 0.8  Doug Johnson (finson@whidbey.com) 

----------

##Firmata Feature: DeviceFeature and Device Drivers

Proposed for addition in Firmata 2.6 or later.

The purpose of this feature is to facilitate arbitrary additions to Firmata capabilities without requiring changes to the underlying protocol.  The feature is implemented with a new FirmataFeature module `DeviceFeature`, a pair of new Sysex commands (`DEVICE_QUERY` and `DEVICE_RESPONSE`), and the concept of a DeviceDriver abstract class with well defined method signatures.  

In effect, `DeviceFeature` uses Firmata as a remote procedure call mechanism.  

The `DeviceFeature` module receives, decodes, and dispatches incoming `DEVICE_QUERY` messages to the appropriate device driver.  The concrete sub-classes of DeviceDriver implement the various capabilities and make them available to callers through the API documented below.  After a request has been processed by the device driver, the `DeviceFeature` module captures the result, encodes, and sends the outgoing `DEVICE_RESPONSE` messages back to the host.

Also note that any other module on the server can use the device driver API directly to access any device capabilities it might require.  In this case, there is no reformatting, encoding, transmission, or other involvement by Firmata, it's just one module calling another directly.

##Introduction

Device Drivers are designed to allow a client-side application to control remote devices attached to a Firmata server.  This is similar to the function of the existing Firmata command set, but at a somewhat higher level of abstraction.

###Terminology

Some terms with specific meanings for this feature are *device*, *logical unit* and *handle*.

- Device.  A device driver can control one or more instances of a specific *device* type.  The capabilities of the driver are determined by the device type, and are equally available to all the instances of the device. Each driver is given a simple name to identify the device type.  For example, DDMCP9808 is the name of a device driver for the MCP9808 temperature sensor.  DDHello is the name of a virtual device driver that deals with "Hello World" type messaging.
- Logical Unit.  Each instance of a device is a *logical unit*.  A device driver can handle one or more logical units, depending on device capabilities and device driver implementation. Each logical unit is given a name based on the device name.  For example, if there are two MCP9808 boards connected to a single server, then they could be referred to as MCP9808:0 and MCP9808:1.
- Handle.  When a logical unit is first opened, a *handle* that uniquely identifies the device and logical unit of interest is returned to the caller for use in future operations.  The handle is valid until the logical unit is closed.

###Device Driver API

The Device Driver API includes the methods `open()`, `read()`, `write()`, `close()`, `processTimerEvent()`, and `reset()`.  The API is intended to be implemented by a device driver module on the server side (Firmata micro) exactly as written.  On the client side (client host), the same API calls should be implemented by a proxy, but there may be small changes dictated by the syntax of the language used for the client.  Client-side proxy device drivers and server-side device drivers always use this API and never compose Firmata messages themselves, instead they rely on the  Firmata library to do that.

In the most common architecture, the device driver implements the main device control code on the server and provides access using the specified API.  A proxy on the client also implements the API signatures, and acts as a bridge to the actual device driver and uses the Device Driver Sysex messages DEVICE\_QUERY and DEVICE\_RESPONSE to control the server side device driver via Firmata, which in turn controls the component(s) using local capabilities.  In this scenario, the server side device driver receives essentially the same calls and parameters as were provided to the proxy on the client.

On the other hand, it is also possible for a device driver author to write the main control code for the client and provide access there using the same API. In this case the client device driver uses existing Firmata Features and commands as necessary to control the remote component(s) directly and according to the data sheet.  In this scenario, the server side Firmata responds to standard Firmata commands as received and there is no specific device driver needed on the server.  This is very similar to the approach used by johnny-five to define Component APIs.

####Device Read and Write Registers

The read and write methods operate based on register numbers.  On an actual device, physical register numbers usually start at 0 and max out at a relatively low value like 16 or 32, depending on the device.  This DeviceDriver API uses a 16-bit signed integer to identify the register of interest, so virtual quantities and actions can be implemented in addition to the actual physical device capabilities.

The register range 0..255 is reserved for actual physical registers on devices.  A device driver should define a name for each physical register that it plans to use.

The register range 256..32767 is reserved for device-specific virtual registers. These registers define the higher level functions of the specific device driver API and are implemented by the author as appropriate.

The register range -1..-255 is reserved for a set of common virtual register numbers that any device driver may implement.  Although it is not required that each device driver implement every common virtual register, there are fairly specific expectations for the meaning of those that are implemented.

The numbers in the range -256..-32768 are reserved for future requirements and should not be used at all at this time.

####Status Return from Methods

Each of the device driver methods returns a signed `int` value to the caller.  If the value is negative, then the call failed and the value is an error code.  If the value is greater than or equal to 0, then the call succeeded.  The meaning of the non-negative value depends on the call.  For example, the `open` method returns a handle for future use and the `read` method returns the number of bytes read.  The details for each method are documented below.

##Method Prototypes

The method prototypes shown below are the primary interface to each Device Driver on the server and, with suitable modifications for language syntax, on the client.

The type identifier `int` is used to indicate a signed integer value of at least 16 bits.  Only the low order 16 bits (the two low order bytes) are significant.  The type identifier `byte` is used to indicate an integer value of at least 8 bits.  Only the low order 8 bits (one byte) are significant.

To the extent practical, the error code values and meanings are taken directly from the Linux/C error codes documented in errno.h and errno-base.h, except that the actual values are negated for use in this application.

All data values visible to the device drivers are encoded to base-64 immediately before transmission by Firmata and decoded to their true values immediately after receipt.  Thus, as far at the device drivers and the clients are concerned, all values are full-width values with no constraints on their magnitude or use of the high order sign bit due to transmission protocols.

---
###Open

    int open(const char* name)
    int open(const char* name, int flags)
    int open(const char* name, int flags, int opt)

**param** (in) `name` Name of the logical unit to open.  UTF-8 encoded, null terminated.

**param** (in) `flags` Flags associated with the open.  **Note:** *Only the low order 4 bits of this parameter are used.*  There are standard values defined that all device drivers may utilize.  Default: 0.

**param** (in) `opt` Option indicator specific to the device driver being addressed by the open.  The meaning (if any) and layout of this 16-bit value are defined by each device driver as (and if) needed.  Default: 0.

**return** *Success*: The newly assigned handle value.  The handle is used in future calls to indicate the device driver and specific device being addressed.  *Error*: error code.

---
###Read

Read information from a register (or virtual register) in the device or device driver.

The method and its parameters are as follows.

    int read(int handle, int flags, int reg, int count, byte *buf)

**param** (in) `handle` The device driver selector value returned by Open in a previous call.

**param** (in) `flags` Flags associated with the read.  **Note:** *Only the low order 4 bits of this parameter are used.*  There are standard values defined that all device drivers may utilize.  Default: 0.

**param** (in) `reg` The register address at which to start reading.

**param** (in) `count` The desired (or maximum) number of bytes to read.

**param** (out) `buf` Pointer to the buffer to receive the data read.  Must be large enough to hold `count` bytes.

**return**  *Success*: The number of bytes actually read.  A short count does not in itself cause an error, since the caller can determine for itself that not everything requested was read which may not actually be an error.  *Error*: error code.

---
###Write

    int write(int handle, int flags, int reg, int count, byte *buf)

**param** (in) `handle` The device driver selector value returned by Open in a previous call.

**param** (in) `flags` Flags associated with the write.  **Note:** *Only the low order 4 bits of this parameter are used.*  There are standard values defined that all device drivers may utilize.  Default: 0.

**param** (in) `reg` The register address at which to start writing.

**param** (in) `count` The desired number of bytes to write.

**param** (in) `buf` Pointer to the buffer containing the data to write.

**return**  *Success*: The number of bytes actually written.  Ordinarily, this will be equal to the requested number of bytes to write.  If it is short due to some detectable device error (eg, physical write failure), then the driver will return an error code (eg, `EIO`).  However, under some unique circumstances for some drivers, it may be reasonable for a short count to occur in which case the driver will return the short count and no error code.  *Error*: error code.

---
###Close

    int close(int handle)
    int close(int handle, int flags)

**param** (in) `handle` The device driver selector value returned by Open in a previous call.  The selected device driver is responsible for deciding what actions if any are needed to "close" the connection.  After a close, the only valid action on the device is another open.

**param** (in) `flags` Flags associated with the close.  **Note:** *Only the low order 4 bits of this parameter are used.*  There are standard values defined that all device drivers may utilize.  Default: 0.

**return**  Success: 0.  *Error*: error code.
