
----------
March 2016 V 0.8  Doug Johnson (finson@whidbey.com) 

----------

# DeviceFeature and Device Drivers - Appendix A: Common Registers and Constants

The document device-driver.md describes the overall architecture of the proposed Arduino Device Driver architecture.

One detail of the design is that device drivers can define virtual register numbers to identify actions that the device driver supports beyond those that can be implemented simply with access to a single hardware register.

This Appendix A document defines the common virtual device registers that work more or less the same way in every device driver that implements them.  Addition virtual registers may be defined by each device driver according to its need.

Similarly, there are various sets of constants that all device drivers can use.  The values and intended meanings are defined below.

##Common Device Registers

The read() and write() methods operate based on register numbers.  On an actual device, physical register numbers usually start at 0 and max out at a relatively low value like 16 or 32, depending on the device.  This DeviceDriver API uses a full 16-bit signed integer to identify the register of interest, so virtual quantities and actions can be implemented in addition to the actual physical device capabilities.

The `reg` parameter is always a signed 16-bit integer value.  Physical device register numbers and device-specific virtual register numbers are non-negative.  The common register numbers defined below are negative.

The **C**ommon **D**evice **R**egister names are in the CDR namespace.

###Method Signatures

    int read(int handle, int flags, int reg, int count, byte *buf)

Read information from a register (or virtual register) in the device or device driver.  

    int write(int handle, int flags, int reg, int count, byte *buf)

Write information to a register (or virtual register) in the device or device driver.  

---
##Register Definitions - Implementation Required

####`CDR::DriverVersion`

#####Get version number and name of the DeviceDriver.

Every device driver must implement this function.  However, since there are macros and base class methods provided to do the work, it is very simple to include in any new device driver.

Version numbers are in the Semantic Versioning format x.y.z-a.b.c  See [semver.org](http://semver.org) for more info.  

The first byte of the return data buffer is the size in bytes of the version identifier packet that follows.  Name strings use UTF-8 encoding and are null-terminated.

In the initial implementation, the size of the version identifier packet is 6 bytes.  The name string immediately follows the version identifier packet and is limited to 128 bytes maximum, including the null terminator.

The size of the receiving buffer should be large enough to hold the 1-byte packet size, a version identifier packet, and a name string (including the null terminator). If the buffer size is not large enough, an error will be returned (`EMSGSIZE`).

*Return data buffer*

     0  version descriptor packet size (6, in this example)
     1  major version (x)
     2  minor version (y)
     3  patch version (z)
     4  pre-release (a)
     5  pre-release (b)
     6  pre-release (c)
     7..n  name string (UTF-8, null terminated)

or

     0  version descriptor packet size (3, in this example)
     1  major version (x)
     2  minor version (y)
     3  patch version (z)
     4..n  name string (UTF-8, null terminated)

---
##Register Definitions - Implementation Optional


####`CDR::LibraryVersion`

#####Get version number and name of the supporting library.

Should use the same Semver format as the driver version information, if at all possible.

---
####`CDR::Intervals`

#####Get/set the microsecond and millisecond timer intervals for the specified open logical unit.

There are two timer intervals for each device driver, one set in microseconds and the other set in milliseconds.  If the microsecond value is non-zero the processTimerEvent(...) method of the driver is called each time the interval elapses.  Similarly, if the millisecond value is non-zero, the same processTimerEvent(...) method is called each time the millisecond interval elapses.  A `timerIndex` value (0 or 1) is provided as an argument to distinguish the two cases.

Base class methods are provided for reading and writing these values, and recognizing when an interval has elapsed.  If the device driver uses one or both of the timers, it just needs to pass the read or write action off to the base class, and also implement the `processTimerEvent() method.  If the device driver does not need this timer capability, it can be ignored completely and the whole process will be handled by the base class.

*Read/Write data buffer format*

     0  microsecond interval (LSB) (ulong32)
     1  microsecond interval
     2  microsecond interval
     3  microsecond interval (MSB)
     4  millisecond interval (LSB) (ulong32)
     5  millisecond interval
     6  millisecond interval
     7  millisecond interval (MSB)

---
####`CDR::Reset`

#####Reset the device driver state for the logical unit number in use.

This is intended to provide a fine-grained (single unit number) reset.  See also the reset() function of the Device Driver API, which is intended to be a broad brush reset (a warm start) of the entire driver.

---
####`CDR::UnitNamePrefix`

#####Get/set the prefix that is recognized by the `open()` method of the implementing device driver.

---
####`CDR::Configure`

#####Get/set the configuration of the logical unit number.

This capability enables a device driver to get/set the configuration of a unit number all in one operation rather than register by register.

---
####`CDR::Stream`

#####Read/Write the data stream associated with a unit number.

For some devices, it is more intuitive to think of the unit number as a simple source or sink of a stream of data packets, rather than a set of data registers.  This Stream register is provided to enable a natural way of expressing that in a `read()` or `write()` call.

---
####`CDR::Manufacturer`

#####Read the manufacturer ID associated with the connected device

If there is a need and a technique to read a manufacturer ID, this is the register to use.  Unfortunately, I haven't found a nice straightforward format for such IDs, and so there's no common implementation provided.

---
####`CDR::DeviceID`

#####Read the device ID associated with the connected device

Same idea and issues as with the manufacturer ID.

---
####`CDR::Debug`

#####Read/Write debugging data

This register is provided as a way to consolidate values and methods that are not part of the intended operation of the device, but are useful enough for debugging that the author wants to include them in the driver.

---
###Common Constants


####Device Action Codes

These are 4-bit values identifying the device driver action to be invoked.  They are only used during communication, and are not visible to either the client library or device driver methods since those libraries have a separate method for each action.

    OPEN    (0x0)
    READ    (0x1)
    WRITE   (0x2)
    CLOSE   (0x3)

####Device Action Flags

Each of the four primary device driver methods take a parameter named `flags`.  These flags act as qualifiers for the primary action.  They are 4-bit values, with the names and meanings shown below.

There is only one set of device action flags shared among all the action methods (open(), read(), write(), and close()).  Presumably the usage of each flag will be similar across the methods, but there is no requirement that it be exactly the same in each method.  Similarly, the same numeric value can be used for entirely different meanings in the different methods. In the latter case, the flags should have different names for clarity, even though the actual numeric values are the same.

<table>
<caption>Device Action Flag Names and Values
<tr><th>Flag Value</th><th>OPEN</th><th>READ</th><th>WRITE</th><th>CLOSE</th></tr>
<tr><td>0x0</td><td><code>NONE</code></td><td><code>NONE</code></td><td><code>NONE</code></td><td><code>NONE</code></td></tr>
<tr><td>0x1</td><td><code>FORCE</code></td><td><code>FORCE</code></td><td><code>FORCE</code></td><td><code>FORCE</code></td></tr>
<tr><td>...</td><td>--</td><td>--</td><td>--</td><td>--</td></tr>
<tr><td>0xC</td><td>--</td><td><code>MILLI_RUN</code></td><td><code>MILLI_RUN</code></td><td>--</td></tr>
<tr><td>0xD</td><td>--</td><td><code>MILLI_STOP</code></td><td><code>MILLI_STOP</code></td><td>--</td></tr>
<tr><td>0xE</td><td>--</td><td><code>MICRO_RUN</code></td><td><code>MICRO_RUN</code></td><td>--</td></tr>
<tr><td>0xF</td><td>--</td><td><code>MICRO_STOP</code></td><td><code>MICRO_STOP</code></td><td>--</td></tr>
</table>

